unit uXSDParser;

interface
uses
  JanXMLParser2,
  uXSDobj;

type
  tLogProcedure = procedure(const s: string) of object;
type
  tXSDParser = class
    FCurrentClass: tClassDef;
    procedure Logout(const s: string);
  private
    FOnLogout: tLogProcedure;
    FClassDefs: tClassDefs;
    procedure parseRestriction(xRest: tJanXMLNode2; const sPref: string);
    procedure parseProperties(xElement, dn: tJanXMLNode2);
    procedure parseSimpleType(xElement, xSimple: tJanXMLNode2; bInClass:
      boolean);
    procedure parseSequence(dn: tJanXMLNode2; bInClass: boolean);
    procedure parseComplextype(xElement, xComplex: tJanXMLNode2; bInClass:
      boolean = false);
    procedure parseAttribute(xAttribute: tJanXMLNode2);
    procedure parseElement(xElement: tJanXMLNode2; bInclass: boolean);
    procedure parseImport(xn: tJanXMLNode2);
  public
    constructor Create;
    destructor Destroy; override;
    procedure parseSchema(xSchema: tJanXMLNode2);
    property OnLogout: tLogProcedure read FOnLogout write FOnLogout;
    property ClassDefs: tClassDefs read FClassDefs;
  end;

implementation
uses
  SysUtils,
  mylib;

procedure tXSDParser.Logout(const s: string);
begin
  if Assigned(FOnLogout) then
    FOnLogout(s);
end;

procedure tXSDParser.parseRestriction(xRest: tJanXMLNode2; const sPref: string);
var
  xEnum: tJanXMLNode2;
  sVal: string;
begin
  Logout('parseRestriction');
  xEnum := xRest.getChildByName(xsEnumeration);
  if Assigned(xEnum) then
    Logout('xEnum: ')
  else
    Logout('xEnum=NIL');
  while Assigned(xEnum) and (xEnum.name = xsEnumeration) do
  begin
    sVal := xEnum.attribute['value'];
    Logout(':' + sVal);
    ClassDefs.AddConst(sPref, sVal);
    xEnum := xEnum.NextSibling;
  end;
end;

procedure tXSDParser.parseSimpleType(xElement, xSimple: tJanXMLNode2; bInClass:
  boolean);
var
  xRest: tJanXMLNode2;
  sElement: string;
  pt: string;
  p: tProperty;
  simple: boolean;
  sNs: string;
  iMax: integer;
begin
  Logout('// parsesimpletype');
  sNs := xElement.attribute[xsmaxOccurs];
  if sNs = xsMunbounded then
    iMax := cUnbounded
  else
    iMax := StrToIntDef(sNs, cScalar);

  if Assigned(xElement) then
    sElement := xElement.attribute[xsename]
  else if Assigned(xSimple) then
    sElement := xSimple.attribute[xsename]
  else
    sElement := '';
  xRest := xSimple.getChildByName(xsrestriction);
  if assigned(xRest) then
  begin
    pt := xRest.attribute[xsRsBase];
    pt := TranslateType(pt);
    simple := true;
    parseRestriction(xRest, sElement);
  end
  else
  begin
    pt := 't' + sElement;
    simple := false;
  end;

  ClassDefs.AddOrdinal(sElement, pt);
  Logout(tabtype + ' t' + sElement + ' = ' + pt + ';');
  if bInClass then
  begin
    p := tProperty.Create(sElement, pt, 'E', '', iMax, simple);
    FCurrentClass.Properties.AddObject(p.name, p);
  end;
end;

procedure tXSDParser.parseProperties(xElement: tJanXMLNode2; dn: tJanXMLNode2);
var
  xn: tJanXMLNode2;
  sType: string;
  sName: string;
  iMax: integer;
  sns: string;
begin // procedure parseProperties(dn: tJanXMLNode2);
  Logout('// parseproperties');

  sName := xElement.attribute[xsename];
  sNs := xElement.attribute[xsmaxOccurs];
  if sNs = xsMunbounded then
    iMax := cUnbounded
  else
    iMax := StrToIntDef(sNs, cScalar);

  xn := dn.getChildByName(xsrestriction);
  if assigned(xn) then
  begin
    sType := xn.attribute[xsRsBase];
    sType := TranslateType(sType);

    parseRestriction(xn, sType);
  end
  else
    sType := 't' + sName;
end; // procedure parseProperties(dn: tJanXMLNode2);

procedure tXSDParser.parseSequence(dn: tJanXMLNode2; bInClass: boolean);
var
  dn2: tJanXMLNode2;
  c: integer;
  i: integer;
begin
  Logout('// parseSequence');

  c := dn.nodes.count;
  for i := 0 to c - 1 do
  begin
    dn2 := tJanXMLNode2(dn.Nodes[i]);
    if dn2.name = xsElement then
      parseElement(dn2, true)
    else if dn2.name = xscomplexType then
      parseComplextype(dn, dn2, bInClass)
    else
      Logout('// --- parseSequence.node: ' + dn2.attribute[xsename]);
  end;
end;

procedure tXSDParser.parseComplextype(xElement, xComplex: tJanXMLNode2;
  bInClass:
  boolean = false);
var
  dn2: tJanXMLNode2;
  c: integer;
  i: integer;
  sName: string;
  sNs: string;
  iMax: integer;

  old: tClassDef;
  p: tProperty;
  b: string;
begin
  Logout('// parseComplextype');
  c := xComplex.nodes.count;

  if bInClass then
    Logout('  (*');

  // complex type needs to create a class header
  if Assigned(xElement) then
  begin
    sName := xElement.attribute[xseName];
    sNs := xElement.attribute[xsmaxOccurs];
    b := 'E';
  end
  else
  begin
    sName := xComplex.attribute[xseName];
    sNs := xComplex.attribute[xsmaxOccurs];
    b := 'A';
  end;
  if sNs = xsMunbounded then
    iMax := cUnbounded
  else
    iMax := StrToIntDef(sNs, cScalar);

  old := FCurrentClass;
  FCurrentClass := tClassDef.Create(sName);
  ClassDefs.AddObject(FCurrentClass.name, FCurrentClass);

  Logout(' ');
  Logout(tabtype);
  Logout(tabClass + 't' + sName + ' = class');
  Logout(tabpublic);

  for i := 0 to c - 1 do
  begin
    dn2 := tJanXMLNode2(xComplex.Nodes[i]);
    if dn2.name = xssequence then
      parseSequence(dn2, bInClass)
    else if dn2.name = xsattribute then
      parseattribute(dn2)
    else
      ;
  end;
  // ends with class footer
  Logout(tabend);
  Logout(' ');
  FCurrentClass := old;

  if bInClass then
  begin
    Logout('  *)');

    // we need to create a property of the current declaration
    p := tProperty.Create(sName, 't' + sName, b, '', iMax, false);
    FCurrentClass.Properties.AddObject(p.name, p);
  end;
end;

procedure tXSDParser.parseAttribute(xAttribute: tJanXMLNode2);
var
  xn: tJanXMLnode2;
  sName: string;
  sType: string;
  p: tProperty;
begin
  Logout('// parseAttribute');
  sName := xAttribute.attribute[xsename];
  sType := xAttribute.attribute[xsetype];
  if sType = '' then
  begin
    xn := xAttribute.getChildByName(xsrestriction, true);
    if assigned(xn) then
    begin
      sType := xn.attribute[xsRsBase];
      parseRestriction(xn, sName);
    end;
  end;
  sType := translateType(sType);

  p := tProperty.Create(sName, sType, 'A', '', cScalar, true);
  FCurrentClass.properties.AddObject(p.name, p);
end;

procedure tXSDParser.parseElement(xElement: tJanXMLNode2; bInClass: boolean);
var
  dn2: tJanXMLNode2;
  c: integer;
  i: integer;
  sElement: string;
  sType: string;
  sRef: string;
  sNs: string;
  iMax: integer;
  old: tClassDef;
  p: tProperty;

  procedure ReferencedType;
  var
    p: tProperty;
    b: string;
    sns: string;
  begin
    // referenced type
    if pos(':', sRef) > 0 then // Namespace -> external reference ?
      sns := CmdSplit(sRef, ':');
    if sns = ClassDefs.NameSpacePrefix then
      b := 'E'
    else
      b := 'X';
    p := tProperty.Create(sRef, 't' + sRef, b, sns, iMax, false);
    FCurrentClass.Properties.AddObject(p.name, p);
  end;

begin // procedure parseElement(dn: tJanXMLNode2);
  sElement := xElement.attribute[xsename];
  sType := xElement.attribute[xsetype];
  sRef := xElement.attribute[xseref];
  sNs := xElement.attribute[xsmaxOccurs];
  if sNs = xsMunbounded then
    iMax := cUnbounded
  else
    iMax := StrToIntDef(sNs, cScalar);

  Logout('// parseElement: ' + sElement + '(' + sType + ')' + sRef);

  dn2 := xElement.getChildByName(xsSimpleType);
  if assigned(dn2) then
    parseSimpleType(xElement, dn2, bInClass)
  else
  begin
    dn2 := xElement.getChildByName(xsComplexType);
    if assigned(dn2) then
      parseComplextype(xElement, dn2, bInClass)
    else
    begin
      if (sElement = '') and (sRef <> '') then
        ReferencedType
      else
      begin
        if not bInClass then
        begin
          Logout('// standallone --------------');
          if sType <> '' then
          begin
            sType := TranslateType(sType);
            ClassDefs.AddOrdinal(sElement, sType);
            Logout(tabtype + ' t' + sElement + ' = ' + sType + ';');
          end
          else
          begin
            old := FCurrentClass;
            FCurrentClass := tClassDef.Create(sElement);
            ClassDefs.AddObject(FCurrentClass.Name, FCurrentClass);

            Logout(' ');
            Logout(tabtype);
            Logout(tabClass + 't' + sElement + ' = class');
            Logout(tabpublic);
            c := xElement.nodes.count;
            for i := 0 to c - 1 do
            begin
              dn2 := tJanXMLNode2(xElement.Nodes[i]);
              if dn2.name = xscomplexType then
                parseComplextype(xElement, dn2, bInClass)
              else if dn2.name = xssequence then
                parseSequence(dn2, bInClass)
              else if dn2.name = xssimpleType then
              else if dn2.name = xsattribute then
                parseAttribute(dn2)
                  ;
              //parseProperties(sElement, dn2);
            end;
            Logout(tabend);
            Logout(' ');
            FCurrentClass := old;
          end;
        end // if bStandallone
        else
        begin
          Logout('// in class --------------');
          if sType <> '' then
          begin
            sType := TranslateType(sType);
            p := tProperty.Create(sElement, sType, 'E', '', iMax, true);
            FCurrentClass.Properties.AddObject(p.name, p);
          end
          else
          begin
            Logout('//   parsing nodelist');
            c := xElement.nodes.count;
            for i := 0 to c - 1 do
            begin
              dn2 := tJanXMLNode2(xElement.Nodes[i]);
              if dn2.name = xscomplexType then
                parseComplextype(xElement, dn2, bInClass)
              else if dn2.name = xssequence then
                parseSequence(dn2, bInClass)
              else if dn2.name = xssimpleType then
                parseSimpleType(xElement, dn2, bInClass)
              else if dn2.name = xsattribute then
                parseAttribute(dn2)
                  ;
              //parseProperties(sElement, dn2);
            end;
          end;
        end; // if bStandallone .. else
      end; // if referenced type .. else
    end; // if complextype .. else
  end; // procedure parseElement(dn: tJanXMLNode2);
end;

procedure tXSDParser.ParseImport(xn: tJanXMLNode2);
var
  sUnit: string;
begin
  sUnit := xn.attribute['schemaLocation'];
  sUnit := ExtractFileName(ExpandFileName(sUnit));
  sUnit := ChangeFileExt(sUnit, '');
  ClassDefs.AddUses(sUnit);
end;

procedure tXSDParser.parseSchema(xSchema: tJanXMLNode2);
var
  c: integer;
  i: integer;
  dn2: tJanXMLNode2;
  tns: string;
  sa: string;
  efd: string;
  afd: string;
begin // procedure parseSchema(dn:tJanXMLNode2)
  ClassDefs.Clear;
  FCurrentClass := nil;

  efd := xSchema.attribute['elementFormDefault'];
  ClassDefs.elementsqualified := efd = cqualified;
  afd := xSchema.attribute['attributeFormDefault'];
  ClassDefs.attributesqualified := afd = cqualified;

  tns := xSchema.attribute['targetNamespace'];
  if tns <> '' then
  begin
    ClassDefs.targetNameSpace := tns;
    // now find the namespace prefix xmlns:...="tns"
    for i := 0 to xSchema.attributecount - 1 do
    begin
      sa := xSchema.attributename[i];
      if pos('xmlns:', sa) = 1 then
      begin
        if tns = xSchema.attribute[i] then
          ClassDefs.NamespacePrefix := copy(sa, 7, length(sa))
        else
          // other namespaces
          ;
      end;
    end;
  end;

  c := xSchema.nodes.count;
  for i := 0 to c - 1 do
  begin
    dn2 := TjanXMLNode2(xSchema.Nodes[i]);
    Logout('// xs:schema ' + dn2.name + ' ' + dn2.text);

    if dn2.name = xselement then
      parseElement(dn2, false)
    else if dn2.name = xsComplexType then
      parseComplextype(nil, dn2)
    else if dn2.name = xsimport then
      parseImport(dn2)
    else if dn2.name = xsSimpletype then
      parseSimpleType(nil, dn2, false)
        ;
  end;
end; // procedure parseSchema(dn:tJanXMLNode2)

constructor tXSDParser.Create;
begin
  FClassDefs := tClassDefs.Create;
end;

destructor tXSDParser.Destroy;
begin
  FClassDefs.Free;
  inherited;
end;

end.

